import re
import bpy


def sw_object(context, sr,
              EX_SELECTED,
              EX_HIDDEN,
              EX_ARMATURE,
              EX_ANIMATION,
              EX_PHYSICS,
              EX_MATERIALS,
              EX_GLOBAL_MATRIX,
              list_mesh,
              list_material,
              list_light,
              list_camera,
              map_webtable,
              index_jsx):
    """
    Object switch; used for creating lists of objects to be exported
    """
    
    # Webtable regex stuff
    wtx_full = "^([Ww]ebtable[s]?)(_[0-9]+)$"
    wtx_full_alt = "^(_brw_[A-Za-z0-9_]+)(_[0-9]+)$"
    wtx_num = "_[0-9]+$"
    reg_full = re.compile(wtx_full)
    reg_full_alt = re.compile(wtx_full_alt)
    reg_num = re.compile(wtx_num)
    list_obj = []
    
    
    def re_coll(list_obj:list, collection:bpy.types.Collection):
        list_obj += collection.objects
        for col in collection.children:
            if col.hide_viewport is True and (EX_HIDDEN is False):
                continue
            re_coll(list_obj, col)
    
    re_coll(list_obj, context.scene.collection)
    
    
    def re_obj(list_obj:list, first:bool):
        up = 0
        for obj in list_obj:
            if (obj.hide_get() is True) and (EX_HIDDEN is False):
                continue
            if (obj.select_get() is False) and EX_SELECTED:
                continue
            
            skip = False # duplicate skipping
            if obj.parent:
                for _obj in list_obj:
                    if obj.parent == _obj:
                        skip = True
                        break
                if skip:
                    continue
            
            if obj.type in {"MESH", "SURFACE", "FONT", "LATTICE"} and obj.data:
                # SWITCH
                # Webtables
                if (reg_full.match(obj.name) or reg_full_alt.match(obj.name)) and obj.type == "MESH":
                    if len(obj.data.vertices) != 4:
                        sr.report({"ERROR"}, f"Webtable data error! Check that your webtable > {obj.name} < only has 4 vertices, and is a plane.")
                        return None
                    
                    ixi = int(reg_num.search(obj.name).group().lstrip("_"))
                    if ixi not in map_webtable.keys():
                        map_webtable[ixi] = obj
                    else:
                        sr.report({"ERROR"}, f"Webtable index error! > {ixi}: {obj.name} <")
                        return None
                
                else: # Meshes
                    list_mesh.append((obj, obj.matrix_world))
                    if first:
                        index_jsx.add_node(obj, obj.matrix_world, EX_GLOBAL_MATRIX, EX_PHYSICS)
                        first = False
                        up += 1
                    else:
                        index_jsx.append_node(obj, obj.matrix_world, EX_GLOBAL_MATRIX, EX_PHYSICS)
                    
                    # Material listing
                    if EX_MATERIALS:
                        done = False
                        if len(obj.data.materials) != 0:
                            for cmat in obj.data.materials:
                                if cmat == None:
                                    continue
                                for rmat in list_material:
                                    if rmat == None:
                                        continue
                                    # if rmat is None:
                                    #     sr.report({"ERROR"}, f"Empty material found in material list of object > {obj} <!")
                                    #     return None
                                    if (cmat != rmat) and (cmat.name == rmat.name):
                                        sr.report({"ERROR"}, f"Material allocated twice in material list of object > {obj} < at materials > {cmat} < and > {rmat} <")
                                        return None
                                    if cmat == rmat:
                                        done = True
                                        break
                                if not done:
                                    list_material.append(cmat)
            
            elif obj.type in {"LIGHT"} and obj.data:
                list_light.append((obj, obj.matrix_world))
                if first:
                    index_jsx.add_node(obj, obj.matrix_world, EX_GLOBAL_MATRIX, EX_PHYSICS)
                    first = False
                    up += 1
                else:
                    index_jsx.append_node(obj, obj.matrix_world, EX_GLOBAL_MATRIX, EX_PHYSICS)
            
            elif obj.type in {"EMPTY"}:
                if first:
                    index_jsx.add_node(obj, obj.matrix_world, EX_GLOBAL_MATRIX, EX_PHYSICS)
                    first = False
                    up += 1
                else:
                    index_jsx.append_node(obj, obj.matrix_world, EX_GLOBAL_MATRIX, EX_PHYSICS)
            
            elif obj.type in {"CAMERA"} and obj.data:
                list_camera.append((obj, obj.matrix_world))
            
            else:
                pass
            
            if obj.children:
                first = True
                re_obj(obj.children, first)
                first = False
        for i in range(0, up):
            index_jsx.pointer_up()
    
    first = True
    re_obj(list_obj, first)
    
    return True
