import bpy
from os import path
from mathutils import Vector
from ..mw_switch import sw_object
from .mw_meta import IndexJSX


def webtable_add(context, sr,
                 *,
                 EX_GLOBAL_MATRIX,
                 size,
                 resolution,
                 make_guide,
                 path_exec_root,
                 ):
    
    map_webtable = {}
    ix_jsx = IndexJSX()
    ix_jsx.init_root()
    
    sw_object(context, sr, False, False, False, False, False, False, EX_GLOBAL_MATRIX, [], [], [], [], map_webtable, ix_jsx)
    
    ix = 1
    for key in map_webtable.keys():
        if int(key) >= ix:
            ix = int(key) + 1
    
    name = f"Webtable_{ix}"
    
    atr = dict()
    rx, ry = resolution.split()
    atr["resolution_x"] = int(rx)
    atr["resolution_y"] = int(ry)
    
    msh = bpy.data.meshes.new(name)
    obj = bpy.data.objects.new(name, msh)
    context.scene.collection.objects.link(obj)
    
    verts = [
        # [-1, -1, 0], [1, -1, 0], [-1, 1, 0], [1, 1, 0]
        Vector((-1, -1, 0)),
        Vector((1, -1, 0)),
        Vector((-1, 1, 0)),
        Vector((1, 1, 0)),
        ]
    
    faces = [[0, 1, 3, 2]]
    
    msh.from_pydata(verts, [], faces)
    
    for k in atr:
        v = atr[k]
    
        obj[k] = v
    
    x, y = size.split()
    
    obj.dimensions = [float(x), float(y), 0]
    
    if make_guide:
        # create guide
        guide = bpy.data.objects.new("wtbg", None)
        context.scene.collection.objects.link(guide)
        guide.empty_display_type = "IMAGE"
        guide.empty_image_side = "FRONT"
        
        # load image (if not already loaded)
        if bpy.data.images.find("webtable.png") is -1:
            src = path.join(path_exec_root, "res", "webtable.png")
            bpy.ops.image.open(filepath=src, relative_path=False, show_multiview=False)
        
        guide.data = bpy.data.images["webtable.png"]
        
        # deselect all
        bpy.ops.object.select_all(action="DESELECT")
        guide.select_set(True)
        obj.select_set(True)
        # parent
        context.view_layer.objects.active = obj
        bpy.ops.object.parent_set(type='OBJECT', keep_transform=False)
    
    context.view_layer.objects.active = obj
    context.view_layer.update()
    
    # TODO MAKE IT SELECTED
    
    return {'FINISHED'}
